// lib/index.js
const axios = require("axios");

class BotDetector {
  constructor(options = {}) {
    this.requestLog = new Map();
    this.botPatterns = options.botPatterns || [
      /bot/i,
      /crawl/i,
      /spider/i,
      /scrape/i,
    ];
    this.suspiciousIps = new Set();
    this.apiEndpoint = options.apiEndpoint || null; // Optional API for IP lookups
  }

  // Method to detect if the request is from a bot
  detect(req) {
    const userAgent = req.headers["user-agent"] || "";
    const ip = req.headers["x-forwarded-for"] || req.socket.remoteAddress;

    // 1. User-Agent check
    if (this.isBotUserAgent(userAgent)) {
      return { isBot: true, reason: "Bot-like User-Agent" };
    }

    // 2. Rate-limiting check
    if (this.isRateLimited(ip)) {
      return { isBot: true, reason: "Excessive request rate" };
    }

    // 3. Check against external IP lookup API (optional)
    if (this.apiEndpoint) {
      return this.checkIp(ip).then((isSuspicious) => {
        if (isSuspicious) {
          return { isBot: true, reason: "Suspicious IP" };
        } else {
          return { isBot: false, reason: "No suspicious activity" };
        }
      });
    }

    return { isBot: false, reason: "No suspicious activity" };
  }

  // Helper: Check if User-Agent looks like a bot
  isBotUserAgent(userAgent) {
    return this.botPatterns.some((pattern) => pattern.test(userAgent));
  }

  // Helper: Check if IP has made too many requests in a short time (basic rate limiting)
  isRateLimited(ip) {
    const now = Date.now();
    const timeWindow = 60000; // 1 minute window
    const maxRequests = 50; // Allow max 50 requests per minute

    if (!this.requestLog.has(ip)) {
      this.requestLog.set(ip, []);
    }

    const requests = this.requestLog
      .get(ip)
      .filter((timestamp) => now - timestamp < timeWindow);

    if (requests.length >= maxRequests) {
      return true;
    }

    requests.push(now);
    this.requestLog.set(ip, requests);
    return false;
  }

  // Helper: Optionally check IP against external API
  async checkIp(ip) {
    try {
      const response = await axios.get(`${this.apiEndpoint}?ip=${ip}`);
      return response.data.isSuspicious || false;
    } catch (error) {
      console.error("Error checking IP:", error);
      return false;
    }
  }
}

module.exports = BotDetector;
